import os
from pprint import pprint

from atptools import DictDefault
from pydantic import BaseModel

from .app_script import AtpRunScript, AtpRunScriptConfig

default_config_path: str = "./atprun.yml"


class AtpRunConfig(BaseModel):
    scripts: dict[str, AtpRunScriptConfig]
    # pipelines: dict[str, list[str]] = {}


class AtpRunMain:
    def __init__(self) -> None:
        self.config_path: str = ""
        self.config_in: DictDefault = DictDefault()
        self.config: AtpRunConfig | None = None
        self.scripts_run: dict[str, AtpRunScript] = {}
        return None

    def _get_configuration_file_path(self, path: str | None) -> str:
        # Load default value
        self.config_path = default_config_path
        # Load environment variable
        if "ATPRUN_CONFIG_PATH" in os.environ:
            self.config_path = os.environ["ATPRUN_CONFIG_PATH"]
        # Load command line argument
        if path is not None and len(path) > 0:
            self.config_path = path

        return self.config_path

    def load_configuration(self, path: str | None) -> None:
        self._get_configuration_file_path(path=path)

        self.config_in.from_file(path=self.config_path)

        self.config = AtpRunConfig.model_validate(
            obj=self.config_in.to_dict(),
            strict=True,
        )

        # prepare scripts run
        if self.config.scripts is not None:
            for name, script in self.config.scripts.items():
                self.scripts_run[name] = AtpRunScript(
                    name=name,
                    script=script,
                )
        return None

    def script_get(self, name) -> AtpRunScript:
        try:
            return self.scripts_run[name]
        except KeyError as err:
            raise ValueError(f"Script '{name}' not found") from err

    def script_run(self, name: str) -> None:
        script: AtpRunScript = self.script_get(name=name)
        script.run()
        return None



# https://typer.tiangolo.com/tutorial/package/

from pprint import pprint

import typer

from .app_main import AtpRunMain

app: typer.Typer = typer.Typer()
atp_run: AtpRunMain = AtpRunMain()


@app.callback()
def callback(
    config_path: str | None = typer.Option(
        default=None,
        help="Configuration file path",
    ),
):
    """
    AtpRun
    """
    # load config file
    atp_run.load_configuration(path=config_path)
    pass


@app.command()
def script(
    name: str,
):
    """
    Run script
    """
    try:
        atp_run.script_run(name=name)
    except ValueError as err:
        typer.secho(
            f"Error: {err}",
            err=True,
            fg=typer.colors.RED,
        )
    except Exception as err:
        typer.secho(
            f"Error [Unexpected]: {err}",
            err=True,
            fg=typer.colors.RED,
        )


if __name__ == "__main__":
    app()



import os
import subprocess

from pydantic import BaseModel


class AtpRunScriptConfig(BaseModel):
    run: str  # TODO: validate len(run) > 0
    name: str | None = None
    # description: str | None = None
    env_var: dict[str, str] | None = None
    # env_files: list[str] | None = None
    # dot_env_group: str | None = None
    # uv_group


class AtpRunScript:
    def __init__(self, name: str, script: AtpRunScriptConfig) -> None:
        self.name: str = name
        self.script: AtpRunScriptConfig = script
        return None

    def _export_env_var(self) -> None:
        if self.script.env_var is not None:
            for key, value in self.script.env_var.items():
                os.environ[key] = value
        return None

    def _command_run(self) -> None:
        command: str = self.script.run
        if len(command) <= 0:
            raise ValueError("No 'run' is empty")
        subprocess.run(
            args=command,
            shell=True,
        )
        return None

    def run(self) -> None:
        self._export_env_var()
        self._command_run()

        return None
